/* This code was created by Jeff Molofee '99 (Ported to Linux/GLX by
 * Patrick Schubert 2000 with the help from lesson 1 basecode for Linux/GLX
 * by Mihael Vrbanec).
 *
 * Cleaned up and Hopefully easier to read version by Jason Schultz (2004)
 *
 * @File:			main.c
 *
 * @Description: 	Main file for the OpenGL/GLX Base Code
 */
 
#include "glx_base.h"


/* initGLScene(void)
 * 
 * Here is where we initialize everything that will be needed in the
 * OpenGL GLX program. Things like glHint, glEnable, any textures, models
 * that will be used with in the program. Etc.
 * Anything that is Initialized here, should be cleaned up in the 
 * cleanGLScene() function
 */
BOOL initGLScene(void)
{
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glFlush();
	
	return True;
}

/* resizeGLScene(void)
 *
 * Resizes the window
 */
void resizeGLScene(void)
{
	float width,height;
	if(GLWin.height == 0)
		GLWin.height = 1;
	if(GLWin.width == 0)
		GLWin.width = 1;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glViewport(0,0,GLWin.width,GLWin.height);
	width = 0.5f;
	height = 0.5f * ((float)(GLWin.height)/GLWin.width);
	glFrustum(-width,width,-height,height,1.0f,100.0f);
	glMatrixMode(GL_MODELVIEW);
}

/* drawGLScene(void)
 *
 * This is our drawing function. What ever we want to/need to draw, will go
 * in this function. 
 */
void drawGLScene(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
}

/* updateGLScene(void)
 *
 * Function that will be used in updating the scene. What ever you you need
 * to do to progress the scene (key presses, timing, rotations, etc), should
 * be put into this function
 */
void updateGLScene(void)
{
}

/* cleanGLScene(void)
 *
 * Here is where we clean up anything we have initialized in the scene.
 * Things like system resources (keymaps, etc), models, textures, etc.
 *
 * Because this is a base code, we don't do anthing with this function
 */
void cleanGLScene(void)
{
}

/* main(void)
 *
 * This is the main function that calls all other functions to make 
 * things run.
 */
int main(void)
{
	/* Create our OpenGL GLX Window 
     * 640x480 24bpp, no fullscreen
	 */
	createGLWindow("NeHe's GLX Base Code", 640, 480, 24, False);
	
	/* Initialize the OpenGL GLX Scene */
	initGLScene();
	
	/* Run the OpenGL GLX Scene */
	runGLScene();
	
	/* Deinitialize OpenGL GLX Scene and clean up anything we have
	 * initialized
	 */
	cleanGLScene();
	
	/* Kill the OpenGL GLX Scene */
	killGLWindow();
	
	return 0;
}
