
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <malloc.h>

#include <pthread.h>
#include <time.h>
#include <sys/time.h>
#include <limits.h>

#ifndef GL_GLEXT_PROTOTYPES
#define GL_GLEXT_PROTOTYPES 1
#endif
#include <GL/gl.h>
#include <GL/glext.h>
#include <GL/glu.h>

#include "gl_shader_tool.h"
#include "glx_base.h"

#define scr_width  800
#define scr_height 600

const unsigned long frames_skip4fps = 20;

static int  ImageWidth  = 0;
static int  ImageHeight = 0;
static int  TextureWidth  = 0;
static int  TextureHeight = 0;
static char ImageFile[160];
static int TextureBpp     =  0;
static GLenum TextureFormat =  GL_NONE; // GL_BGRA; // GL_RGBA
static char TextureFormat_s[80];

GLhandleARB glsl_program = 0;
GLhandleARB vertex_shader = 0;
GLhandleARB fragment_shader = 0;

char vertex_shader_file[160];
char fragment_shader_file[160];

int my_brightness_uniform_location=-1;

GLbyte *pData;
GLuint texName = 0;
GLuint texNameCT = 0;
struct timeval  mtime1, mtime2;

void cleanGLScene(void)
{
  if( pData ) {
    free( pData );
  }
  (void) gl_delete_shader( &glsl_program, &vertex_shader, &fragment_shader );
}

static int64_t getusecs(void)
{
    struct timeval tv;
    int64_t res=-1;

    if ( 0==gettimeofday(&tv, NULL) )
    {
        res  = tv.tv_sec; 
        res *= 1000000L;    
        res += tv.tv_usec;
    }

    return res;
}

void *  allocPixelBuffer( long pixelBufferSize)
{
   void * pixelBuffer = NULL;

   printf("info: using memalign for memory, size %ld, align %ld\n", pixelBufferSize, sysconf(_SC_PAGESIZE));
   fflush(NULL);

   pixelBuffer = memalign(sysconf(_SC_PAGESIZE), pixelBufferSize);
   if ( !pixelBuffer )
   {
	     printf("error: memalign of %ld bytes with alignment %ld failed\n", pixelBufferSize, sysconf(_SC_PAGESIZE));
         exit(-1);
   }
   return pixelBuffer;
}

static GLfloat vertices[] = {
  0.00f, 0.00f,
  0.00f, 1.00f,
  1.00f, 1.00f,
  0.00f, 0.00f,
  1.00f, 1.00f,
  1.00f, 0.00f };

static GLfloat texCoords[] = {
  0.0f, 0.0f,
  0.0f, 1.0f,
  1.0f, 1.0f,
  0.0f, 0.0f,
  1.0f, 1.0f,
  1.0f, 0.0f };

/* new window size or exposure */
void resizeGLScene(void)
{
   glViewport(0, 0, (GLint) GLWin.width, (GLint) GLWin.height);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glFrustum( -1.0, 1.0, -1.0, 1.0, 0.0, 0.0);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}
   
void drawGLScene(void)
{
  static unsigned long frames=0;
  static int64_t startTime=0, lastTime=0;
  static GLfloat brightness = 1.0f, brightness_incr=-0.1f;
  int64_t currentTime, t0;
  double fps=0.0, dt=0.0;

  t0 = getusecs();

  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  glColor3f(1.0f, 1.0f, 1.0f);

  glDrawArrays(GL_TRIANGLES, 0, 6);

  pfn_glUniform1fARB(my_brightness_uniform_location, brightness);
  brightness+=brightness_incr;
  if(brightness<=0.0f)
  {
	brightness=0.1f;
	brightness_incr*=-1.0f;
  } else if(brightness>=1.0f)
  {
	brightness=0.9f;
	brightness_incr*=-1.0f;
  }

  currentTime = getusecs();
  frames++;
  if(startTime==0 || frames==frames_skip4fps)
  {
    if (frames==frames_skip4fps)
        printf("fps reset - skipped %lu frames\n", frames_skip4fps);
    else
        printf("fps reset - start\n");
    startTime=currentTime;
  } else if ( frames>frames_skip4fps )
  {
     dt = (double)( ((currentTime-lastTime)/10000L) ) / 100.0 ; // in decimale sec.
     if( dt >= 1.0 )
     {
	     dt = (double)( ((currentTime-startTime)/10000L) ) / 100.0 ; // in decimale sec.
	     fps = (double)(frames-frames_skip4fps) / dt;
	     printf("%6.2lfs, %4lu F %5.2lf FPS\n", dt, frames, fps);
	     lastTime = currentTime;
     }
  }
}

void updateGLScene(void)
{
}

BOOL initGLScene(void)
{
  GLint fileSize, textureSize;
  int e_x, e_y, haveShader;

  pData = NULL;

  gl_shader_loadExtention((PFNGLGETPROCADDRESSOS)glXGetProcAddress);

  (void) glGetError(); // flush previous error ..

  haveShader = !gl_compile_shaderfiles( &glsl_program, &vertex_shader, &fragment_shader,
				        vertex_shader_file,
				        fragment_shader_file,
				        1 /* verbose */
				       );

  glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );

  glEnable( GL_DEPTH_TEST );

  //glAlphaFunc(GL_GREATER, 0.5);
  //glEnable(GL_ALPHA_TEST);

  /* achieve the 2**e_x>=imageWidth, 2**e_y>=imageHeight */
  e_x=0; TextureWidth=1;
  while (TextureWidth<ImageWidth)
  { TextureWidth*=2; e_x++; }

  e_y=0; TextureHeight=1;
  while (TextureHeight<ImageHeight)
  { TextureHeight*=2; e_y++; }

  printf("Size Screen:        %dx%d\n", scr_width, scr_height);
  printf("Size Image/Texture: %dx%d -> %dx%d\n", ImageWidth, ImageHeight, TextureWidth, TextureHeight);

  textureSize = TextureWidth * TextureHeight * TextureBpp/8;
  pData = (GLbyte*)allocPixelBuffer(textureSize);
  memset(pData, 0, textureSize); // clear memory

  fileSize = ImageWidth * ImageHeight * TextureBpp/8;
  FILE *fp = fopen(ImageFile, "rb");
  if (!fp)
  {
      fprintf(stderr, "Error opening sample.ATITC file\n");
      cleanGLScene();
  }
  fread(pData, fileSize, 1, fp);
  fclose(fp);

  if(haveShader)
  {
	int texture_unit = 0;
	int uniform_location;
	uint8_t  * color_map = NULL;
	int        color_map_wd = 256; // 256 * ( B G R )
	int        color_map_ht =   3;
	int        color_map_sz = color_map_ht*color_map_wd;
	int        color_map_wd_tx = 1024;
	unsigned int x,y,i;

	color_map = calloc(color_map_sz, sizeof(uint8_t));
	for(y=0; y<color_map_ht && y<8; y++)
	{
		for(x=0; x<color_map_wd; x++)
		{
			i = y*color_map_wd+x;
			switch(y)
			{
			    case 0: // BLUE, or 1:1
				color_map[i]= x;
				break;
			    case 1: // GREEN, or reverse ~c
				color_map[i]= 255-x;
				break;
			    default: // RED, or halfbright 1/c
				color_map[i]= x / 2;
				break;
			}

			if(i%8==0||x==0) printf("\n%3d/%3d: %3d: ", x,y, i);
			printf("0x%X ", color_map[i]);
		}
		printf("\n");
	}
	printf("\n");

	(void) glGetError(); // flush previous error ..
	printf("Using shader ..\n");

	// Use The Program Object Instead Of Fixed Function OpenGL
	pfn_glUseProgramObjectARB(glsl_program);

	glEnable(GL_TEXTURE_2D);
	texture_unit=0;

	uniform_location = pfn_glGetUniformLocationARB(glsl_program, "my_color_texture");
	glActiveTexture(GL_TEXTURE0 + texture_unit);
	pfn_glUniform1iARB(uniform_location, texture_unit);

	glGenTextures(1, &texName);
	glBindTexture(GL_TEXTURE_2D, texName);
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, TextureWidth, TextureHeight,
		     0, TextureFormat, GL_UNSIGNED_BYTE, NULL );
	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, ImageWidth, ImageHeight, 
		     TextureFormat, GL_UNSIGNED_BYTE, pData );


	texture_unit=1;

	uniform_location = pfn_glGetUniformLocationARB(glsl_program, "my_colortable_texture");
	glActiveTexture(GL_TEXTURE0 + texture_unit);
	pfn_glUniform1iARB(uniform_location, texture_unit);

	glGenTextures(1, &texNameCT);
	glBindTexture(GL_TEXTURE_2D, texNameCT);
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	glTexImage2D(GL_TEXTURE_2D, 0, 1, color_map_wd_tx, 1,
                 0, GL_RED, GL_UNSIGNED_BYTE, NULL );

	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, color_map_wd*color_map_ht, 1,
		     GL_RED, GL_UNSIGNED_BYTE, color_map );

	free(color_map);

	glDisable(GL_TEXTURE_2D);

	my_brightness_uniform_location = pfn_glGetUniformLocationARB(glsl_program, "my_brightness");
	pfn_glUniform1fARB(my_brightness_uniform_location, 1.0f);

  } else {
	exit(-1);
  }

  {
    GLfloat ImgScrHeight = (double)ImageHeight/(double)scr_height;
    GLfloat ImgScrWidth  = (double)ImageWidth/(double)scr_width;

    GLfloat ImgTexHeight = (double)ImageHeight/(double)TextureHeight;
    GLfloat ImgTexWidth  = (double)ImageWidth/(double)TextureWidth;

    printf("Text Coord: 0/0 .. %lf/%lf\n", ImgTexWidth, ImgTexHeight);
    printf("Vert Coord: 0/0 .. %lf/%lf\n", ImgScrWidth, ImgScrHeight);

    printf("Size Image/Texture: %dx%d -> %dx%d\n", ImageWidth, ImageHeight, TextureWidth, TextureHeight);
    
     vertices[0] = -ImgScrWidth;     vertices[1] = -ImgScrHeight;
     vertices[2] = -ImgScrWidth;     vertices[3] =  ImgScrHeight;
     vertices[4] =  ImgScrWidth;     vertices[5] =  ImgScrHeight;
     vertices[6] = -ImgScrWidth;     vertices[7] = -ImgScrHeight;
     vertices[8] =  ImgScrWidth;     vertices[9] =  ImgScrHeight;
     vertices[10]=  ImgScrWidth;     vertices[11]= -ImgScrHeight;

     texCoords[3] = ImgTexHeight;
     texCoords[5] = ImgTexHeight;
     texCoords[9] = ImgTexHeight;
     texCoords[4] = ImgTexWidth;
     texCoords[8] = ImgTexWidth;
     texCoords[10] = ImgTexWidth;

     glEnableClientState(GL_VERTEX_ARRAY);
     glEnableClientState(GL_TEXTURE_COORD_ARRAY);
     glVertexPointer(2, GL_FLOAT, 0, vertices);
     glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
  }

  glFlush();
  return True;
}

static void printHelp()
{
   printf("HeLp:\n");
   printf("    -texfile   <texfile> \n");
   printf("    -texwidth  <width> \n");
   printf("    -texheight <width> \n");
   printf("    -texbpp    <width>  24 or 32 bits per pixel\n");
   printf("    -texformat <rgb|bgr>\n");
   printf("    -vshader   <shaderfile>\n");
   printf("    -fshader   <shaderfile>\n");
}

int main(int argc, char *argv[])
{
   GLboolean printInfo = GL_FALSE;
   int i, texparam=0;

   strcpy(vertex_shader_file, "shader/textmap01.vsh");
   strcpy(fragment_shader_file, "shader/textmap01.fsh");

   for (i = 1; i < argc; i++) {
      if (strcmp(argv[i], "-info") == 0) {
         printInfo = GL_TRUE;
      }
      else if (strcmp(argv[i], "-texwidth") == 0) {
         if ( i+1 < argc )
         {
             i++;
             ImageWidth = atoi(argv[i]);
             texparam++;
         } else {
	        fprintf(stderr, "Error: incomplete texwidth parameter:%s\n", argv[i]);
            exit(-1);
         }
      }
      else if (strcmp(argv[i], "-texheight") == 0) {
         if ( i+1 < argc )
         {
             i++;
             ImageHeight = atoi(argv[i]);
             texparam++;
         } else {
	        fprintf(stderr, "Error: incomplete texheight parameter:%s\n", argv[i]);
            exit(-1);
         }
      }
      else if (strcmp(argv[i], "-texbpp") == 0) {
         TextureBpp=0;
         if ( i+1 < argc )
         {
             i++;
             TextureBpp = atoi(argv[i]);
             texparam++;
         }
         if (TextureBpp==0 || !(TextureBpp==24 || TextureBpp==32) )
         {
	        fprintf(stderr, "Error: incomplete texbpp parameter:%s (must be 24 or 32)\n", argv[i]);
            exit(-1);
         }
      }
      else if (strcmp(argv[i], "-texformat") == 0) {
         TextureFormat =  GL_NONE;
         if ( i+1 < argc )
         {
             i++;
             if ( !strcmp(argv[i], "rgb") )
             {
                TextureFormat = GL_RGB;
                strcpy(TextureFormat_s, "GL_RGB");
             } else if ( !strcmp(argv[i], "bgr") )
             {
                TextureFormat = GL_BGR;
                strcpy(TextureFormat_s, "GL_BGR");
             }
             texparam++;
         } 
         if ( TextureFormat==GL_NONE || !(TextureFormat==GL_BGR || TextureFormat==GL_RGB) )
         {
	        fprintf(stderr, "Error: incomplete texformat parameter:%s (can be rgb or bgr, with alpha, if bpp is 32)\n", argv[i]);
            exit(-1);
         }
      }
      else if (strcmp(argv[i], "-texfile") == 0) {
         if ( i+1 < argc )
         {
             i++;
             strncpy(ImageFile, argv[i], 159);
             texparam++;
         } else {
	        fprintf(stderr, "Error: incomplete texfile parameter:%s\n", argv[i]);
            exit(-1);
         }
      }
      else if (strcmp(argv[i], "-vshader") == 0) {
         if ( i+1 < argc )
         {
             i++;
             strncpy(vertex_shader_file, argv[i], 159);
         } else {
	        fprintf(stderr, "Error: incomplete vshader parameter:%s\n", argv[i]);
            exit(-1);
         }
      }
      else if (strcmp(argv[i], "-fshader") == 0) {
         if ( i+1 < argc )
         {
             i++;
             strncpy(fragment_shader_file, argv[i], 159);
         } else {
	        fprintf(stderr, "Error: incomplete fshader parameter:%s\n", argv[i]);
            exit(-1);
         }
      }
      else {
         printf("Warning: unknown parameter: %s\n", argv[i]);
         printHelp();
         exit(-1);
     }
   }

   if ( texparam!=5 )
   {
        fprintf(stderr, "Error: incomplete texture parameter, you have to specify texwidth, texheight, texbpp, texfile and texformat\n");
        printHelp();
        exit(-1);
   }

   if ( TextureBpp==32 ) 
   {
        switch ( TextureFormat ) 
        {
            case GL_RGB:
                TextureFormat=GL_RGBA;
                strcpy(TextureFormat_s, "GL_RGBA");
                break;
            case GL_BGR:
                TextureFormat=GL_BGRA;
                strcpy(TextureFormat_s, "GL_BGRA");
                break;
        }
   }

   printf("Texture: %s: %dx%dx%d %s\n", 
        ImageFile, ImageWidth, ImageHeight, TextureBpp, TextureFormat_s);

   printf("vertex shader: %s\n", vertex_shader_file);
   printf("fragment shader: %s\n", fragment_shader_file);

   createGLWindow("demo", scr_width, scr_height, 24, False);

   if (printInfo) {
      printf("GL_RENDERER   = %s\n", (char *) glGetString(GL_RENDERER));
      printf("GL_VERSION    = %s\n", (char *) glGetString(GL_VERSION));
      printf("GL_VENDOR     = %s\n", (char *) glGetString(GL_VENDOR));
      printf("GL_EXTENSIONS = %s\n", (char *) glGetString(GL_EXTENSIONS));
   }

   initGLScene();

   runGLScene();

   cleanGLScene();
   killGLWindow();
   return 0;
}

