/* This code was created by Jeff Molofee '99 (Ported to Linux/GLX by
 * Patrick Schubert 2000 with the help from lesson 1 basecode for Linux/GLX
 * by Mihael Vrbanec).
 *
 * Cleaned up and Hopefully easier to read version by Jason Schultz (2004)
 *
 * @File:			main.c
 *
 * @Description: 	Main file for the OpenGL/GLX Base Code
 */
 
#include "gl_shader_tool.h"
#include "glx_base.h"

GLhandleARB glsl_program = 0;
GLhandleARB vertex_shader = 0;
GLhandleARB fragment_shader = 0;

const char * vertex_shader_file = "shader/null.vsh";
const char * fragment_shader_file = "shader/null.fsh";
//const char * vertex_shader_file = "shader/error.vsh";
//const char * fragment_shader_file = "shader/error.fsh";

const GLcharARB * vertex_shader_src = 
" void main(){ "
"	gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;"
"}";

const GLcharARB * fragment_shader_src = 
"void main(){"
"    gl_FragColor = vec4(1.0, 1.0, 0.0, 0.0);"
"}";


/* initGLScene(void)
 * 
 * Here is where we initialize everything that will be needed in the
 * OpenGL GLX program. Things like glHint, glEnable, any textures, models
 * that will be used with in the program. Etc.
 * Anything that is Initialized here, should be cleaned up in the 
 * cleanGLScene() function
 */
BOOL initGLScene(void)
{
	int err;

	gl_shader_loadExtention((PFNGLGETPROCADDRESSOS)glXGetProcAddress);

	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glFlush();
	
#if 1
	err = gl_compile_shaderfiles( &glsl_program, &vertex_shader, &fragment_shader,
				      vertex_shader_file,
				      fragment_shader_file,
				      1 /* verbose */
				    );
#else
	err = gl_compile_shader( &glsl_program, &vertex_shader, &fragment_shader,
				 vertex_shader_src,
				 fragment_shader_src
			       );
#endif

         printf("gl_compile_shader: err=%d\n", err);

    if(!err)
    {
	// Use The Program Object Instead Of Fixed Function OpenGL
	pfn_glUseProgramObjectARB(glsl_program);
    }

	return True;
}

void resizeGLScene(void)
{
   glViewport(0, 0, (GLint) GLWin.width, (GLint) GLWin.height);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glFrustum( -1.0, 1.0, -1.0, 1.0, 1.0, 1.0);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   (void)glGetError();
}
   
/* drawGLScene(void)
 *
 * This is our drawing function. What ever we want to/need to draw, will go
 * in this function. 
 */
void drawGLScene(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	  glColor3f(1.0f, 1.0f, 1.0f);

	    glBegin(GL_QUADS);
		/* top of cube */
		glVertex3f(0.0f, 0.0f, 0.0f);
		glVertex3f(1.0f, 0.0f, 0.0f);
		glVertex3f(1.0f, 1.0f, 0.0f);
		glVertex3f(0.0f, 1.0f, 0.0f);
	    glEnd();
}

/* updateGLScene(void)
 *
 * Function that will be used in updating the scene. What ever you you need
 * to do to progress the scene (key presses, timing, rotations, etc), should
 * be put into this function
 */
void updateGLScene(void)
{
}

/* cleanGLScene(void)
 *
 * Here is where we clean up anything we have initialized in the scene.
 * Things like system resources (keymaps, etc), models, textures, etc.
 *
 * Because this is a base code, we don't do anthing with this function
 */
void cleanGLScene(void)
{
    (void) gl_delete_shader( &glsl_program, &vertex_shader, &fragment_shader );
}

/* main(void)
 *
 * This is the main function that calls all other functions to make 
 * things run.
 */
int main(void)
{
	/* Create our OpenGL GLX Window 
     * 640x480 24bpp, no fullscreen
	 */
	createGLWindow("NeHe's GLX Base Code", 640, 480, 24, False);
	
	/* Initialize the OpenGL GLX Scene */
	initGLScene();
	
	/* Run the OpenGL GLX Scene */
	runGLScene();
	
	/* Deinitialize OpenGL GLX Scene and clean up anything we have
	 * initialized
	 */
	cleanGLScene();
	
	/* Kill the OpenGL GLX Scene */
	killGLWindow();
	
	return 0;
}
